#import "EsptouchPlugin.h"
#import "ESPTouchTask.h"
#import "ESPTouchResult.h"
#import "ESP_NetUtil.h"
#import "ESPTouchDelegate.h"
#import "ESPAES.h"
#import "AFNetworking.h"
#import "ESPTools.h"

@interface EspTouchDelegateImpl : NSObject<ESPTouchDelegate>

@end

@implementation EspTouchDelegateImpl

-(void) dismissAlert:(UIAlertView *)alertView
{
    [alertView dismissWithClickedButtonIndex:[alertView cancelButtonIndex] animated:YES];
}

-(void) showAlertWithResult: (ESPTouchResult *) result
{
    NSString *title = nil;
    NSString *message = [NSString stringWithFormat:@"%@ is connected to the wifi" , result.bssid];
    NSTimeInterval dismissSeconds = 3.5;
    UIAlertView *alertView = [[UIAlertView alloc]initWithTitle:title message:message delegate:nil cancelButtonTitle:nil otherButtonTitles:nil];
    [alertView show];
    [self performSelector:@selector(dismissAlert:) withObject:alertView afterDelay:dismissSeconds];
}

-(void) onEsptouchResultAddedWithResult: (ESPTouchResult *) result
{
    NSLog(@"EspTouchDelegateImpl onEsptouchResultAddedWithResult bssid: %@", result.bssid);
    dispatch_async(dispatch_get_main_queue(), ^{
        [self showAlertWithResult:result];
    });
}

@end


@implementation EsptouchPlugin
{
    NSCondition *_condition;
    ESPTouchTask *_esptouchTask;
    EspTouchDelegateImpl *_esptouchDelegate;
}

- (instancetype)init
{
    self = [super init];
    if (self) {
        _condition = [[NSCondition alloc]init];
        _esptouchDelegate = [[EspTouchDelegateImpl alloc]init];
    }
    return self;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"flutteresptouchplugin"
            binaryMessenger:[registrar messenger]];
  EsptouchPlugin* instance = [[EsptouchPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS" stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  }else if([@"getWifiInfo" isEqualToString:call.method]){
    result([self getWifiInformation]);
  }else if ([@"startConnect" isEqualToString:call.method]){
      NSDictionary *argument = (NSDictionary *)call.arguments;
      NSString *ssid = argument[@"ssid"];
      NSString *bssid = argument[@"bssid"];
      NSString *pwd = argument[@"password"];
      dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
          result([self startConnectionSSID:ssid Bssid:bssid Password:pwd]);
      });
  }else if ([@"cancelConnect" isEqualToString:call.method]){
      [self cancel];
  }
  else {
    result(FlutterMethodNotImplemented);
  }
}

- (NSDictionary *)getWifiInformation{
  NSMutableDictionary *wifiDic = [NSMutableDictionary dictionaryWithCapacity:0];
  wifiDic[@"ssid"] = ESPTools.getCurrentWiFiSsid;
  wifiDic[@"bssid"] = ESPTools.getCurrentBSSID;
  return wifiDic;
}

- (NSArray *)startConnectionSSID:(NSString *)ssid Bssid:(NSString *)bssid Password:(NSString *)password{
    [_condition lock];
    _esptouchTask = [[ESPTouchTask alloc]initWithApSsid:ssid andApBssid:bssid andApPwd:password];
    // set delegate
    [_esptouchTask setEsptouchDelegate:_esptouchDelegate];
    [_esptouchTask setPackageBroadcast:YES];
    [_condition unlock];
//    ESPTouchResult *ESPTR = self._esptouchTask.executeForResult;
    NSArray * esptouchResults = [_esptouchTask executeForResults:1];
    NSLog(@"executeForResult() result is: %@",esptouchResults);
    return esptouchResults;
}


- (void) cancel
{
    [_condition lock];
    if (_esptouchTask != nil)
    {
        [_esptouchTask interrupt];
    }
    [_condition unlock];
    NSLog(@"do cancel action...");
}

@end


